/**
 * 
 */
package org.springblade.cube.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.dto.BrowserParam;
import org.springblade.cube.dto.BrowserRequest;
import org.springblade.cube.dto.BrowserResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeTableColumnService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/cube/browser")
@Api(value = "数据集接口", tags = "数据集数据浏览接口")
public class CubeDataBrowserController extends BladeController {
	
	@Autowired
	private QuerySqlService querySqlService;

	@Autowired
	private CubeDataService cubeDataService;
	
	@Autowired
	private CubeTableColumnService cubeTableColumnService;
	
	@Autowired
	private IReportDatasourceService reportDatasourceService;
	
	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	@Autowired
	private DialectContext dialectContext;
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/{cubeId}")
	@ApiOperation(value = "浏览数据接口")
	public R<BrowserResult> browser(@PathVariable Integer cubeId, @RequestBody BrowserRequest browserRequest) {
		//获取分页model和查询参数model
		List<BrowserParam> paramList = browserRequest.getParamList();
		Query query = browserRequest.getPageQuery();
		
		CubeData cubeData = cubeDataService.getById(cubeId);
		if(cubeData == null) {
			return R.fail("数据集不存在");
		}
		//检查数据集是否有权限
		boolean checkCubePermRet = cubeDataPermissionService.checkCubePermForUser(cubeId, cubeData.getCreateUserId(), getUser());
		if(!checkCubePermRet) {
			return R.fail("数据集没有访问权限");
		}
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
		String fromSql = cubeData.getQuerySql();
		//构造Select模块
		List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeId));
		//初始化别名
		for(int i = 0; i < columnList.size(); i++) {
			columnList.get(i).setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
		}
		//查询SQL
		List<String> selectSqlList = new ArrayList<String>();
		//条件SQL
		List<String> conditionSqlList = new ArrayList<String>();
		
		Map<String, List<Object>> extraParams = CubeConstants.mergeAllSysParams(AuthUtil.getUser(), DateFormatEnum.yyyy_MM_ddhh$mm$ss, new HashMap<>());
		
		for(CubeTableColumn column : columnList) {
			//计算字段包含聚合函数，不做查询
			if(Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 1) {
				continue;
			}
			//处理全局参数变量
			String columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
			String selectColumnName = columnName, queryColumnName = columnName;
			// 格式化日期
			if (column.getTimeDataFormat() != null
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
				queryColumnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), queryColumnName);
				selectColumnName = column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss ? queryColumnName : selectColumnName;
			}
			// 格式化数字
			if (column.getNumberDataFormat() != null
					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
				//数字格式化函数
				selectColumnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
				//select显示拼接百分号
				if(NumberFormatEnum.isPercentFormat(column.getNumberDataFormat())) {
					selectColumnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(FunctionMethodEnum.NUMBER_JOINT_PERCENT, selectColumnName, column.getNumberDataFormat().getOracleFmFormat());
				}
			}
			selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
			//是否包含在查询参数中
			BrowserParam browserParam = CollectionUtil.isNotEmpty(paramList) ? paramList.stream().filter(item -> item.getCubeColumnId().equals(column.getId())).findFirst().orElse(null) : null;
			if(browserParam != null) {
				String conditionSql = browserParam.getFunction().handle(queryColumnName, column.getViewDataType(), browserParam.getValues(), dsTypeEnum);
				if(StrUtil.isNotEmpty(conditionSql)) {
					conditionSqlList.add(conditionSql);
				}
			}
		}
		StringBuilder querySqlBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList)).append(" from ").append(fromSql);
		//解析数据集全局过滤器
		if(StrUtil.isNotBlank(cubeData.getFilterJson())) {
			PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
			//解析SQL
			Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class), 
					JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, extraParams);
			if(StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
				conditionSqlList.add(whereHavingSqlTuple.getT1());
			}
		}
		//行权限
		CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeId, getUser());
		if(StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
			conditionSqlList.add(cubePermParseDto.getFilterSql());
		}
		//拼接查询条件
		if(!conditionSqlList.isEmpty()) {
			querySqlBuilder.append(" where " + String.join(" AND ", conditionSqlList));
		}
		
		//ID和别名Map
		Map<Integer, String> columnDict = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));
		
		//设置OrderBy
		Integer[] ascArr = Func.toIntArray(query.getAscs());
		query.setAscs(Stream.of(ascArr).filter(x -> x > 0).map(t -> columnDict.get(t)).collect(Collectors.joining(",")));
		Integer[] descArr = Func.toIntArray(query.getDescs());
		query.setDescs(Stream.of(descArr).filter(x -> x > 0).map(t -> columnDict.get(t)).collect(Collectors.joining(",")));
		
		//分页查询
		IPage<Map<String, Object>> pageResult = dynamicTableService.selectPage(Condition.getPage(query), reportDatasource, querySqlBuilder.toString());
		List<Map<String, Object>> resultList = pageResult != null ? pageResult.getRecords() : null;
		//列权限，删除掉不显示的列
		_formatExcludeColumn(resultList, columnList, cubePermParseDto.getExcludeColumnIdList());
		return R.data(new BrowserResult(pageResult, columnDict), querySqlBuilder.toString());
	}
	
	/**
	 * 列权限校验
	 * @param fieldDict
	 * @param resultList
	 * @param columnList
	 * @param excludeColumnIdList
	 */
	void _formatExcludeColumn(List<Map<String, Object>> resultList, List<CubeTableColumn> columnList, Set<Integer> excludeColumnIdList) {
		//列权限，删除掉不显示的列
		if(CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
			Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));
			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
			if(CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
				//格式化数据
				for(Map<String, Object> jsonObj : resultList) {
					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
				}
			}
		}
	}
}
