/**
 * 
 */
package org.springblade.visual.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.dto.PreviewDetailResult;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.*;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.microsoft.sqlserver.jdbc.StringUtils;

import cn.hutool.core.collection.ListUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

import javax.el.ELClass;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("visual/component")
@Api(value = "模版组件", tags = "模版组件接口")
public class VisualComponentCotroller extends BladeController{

	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	@Autowired
	private QuerySqlService querySqlService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;
	
	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualComponent")
	public R<VisualComponent> detail(VisualComponent visualComponent) {
		VisualComponent detail = visualComponentService.getOne(Condition.getQueryWrapper(visualComponent));
		return R.data(detail);
	}

	/**
	 * 根据大屏id查询组件列表
	 * @param visualId
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "查询组件列表", notes = "传入大屏id visualId")
	public R<List<VisualComponent>> queryComponentListByVisualId(Integer visualId){

		return R.data(visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId,visualId)));
	}
	
	/**
	 * 详情
	 */
	@PostMapping("/delete")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(visualComponentService.removeByIds(Func.toIntList(ids)));
	}
	
	/**
	 * 
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/detailPreview")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "清单组件数据预览", notes = "传入ComponentDto")
	public R<PreviewDetailResult> detailPreview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		// 这里的数据不需要进行时间的补全
		componentDto.setIsDateComplete(false);
		componentDto.setFilterColum(" ");
		R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser(), ComponentTypeEnum.DETAIL);
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		//页码和每页条数
		int pageNum = componentDto.getPageNum() == null || componentDto.getPageNum() <= 0 ? 1 : componentDto.getPageNum();
		int limitVal = component.getLimitVal() == null || component.getLimitVal() <= 0 ? CubeConstants.DEFAULT_PREVIEW_DATA_SIZE : component.getLimitVal(); 
		IPage<Map<String, Object>> pageData = dynamicTableService.selectPage(new Page<>(pageNum, limitVal), reportDatasource, querySql);
		PreviewDetailResult previewDetailResult = null;
		if(pageData != null) {
			List<Map<String, Object>> resultList = pageData.getRecords();
			//列权限，删除掉不显示的列
			_formatExcludeColumn(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList());
			//封装数据
			previewDetailResult = new PreviewDetailResult(pageData.getTotal(), pageData.getPages(), pageData.getSize());
			previewDetailResult.setDataList(resultList);
			previewDetailResult.setFieldDict(fieldDict);
			previewDetailResult.setCubeName(querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null);
			previewDetailResult.setPermStatus(true);
		}
		return R.data(previewDetailResult, querySql);
	}
	
	/**
	 * 
	 * @param value
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/preview")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "数据预览", notes = "传入ComponentDto")
	public R<PreviewResult> preview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);

		// 判断当前的查询是否需要补齐数据
		String filterColum = getFilterColum(componentDto.getColumnList());
		if (StrUtil.isNotEmpty(filterColum)){
			List<Object> selectTimes = getSelectTime(componentDto.getFilterList(),filterColum);
			if (selectTimes.size() < 1){
				return R.fail("请设置查询的时间范围...");
			}
			componentDto.setIsDateComplete(true);
			componentDto.setFilterColum(filterColum);
			componentDto.setSelectTime(selectTimes);
		} else {
			componentDto.setIsDateComplete(false);
			componentDto.setFilterColum("");
		}

		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		if (component == null || component.getCubeDataId() < 1){
			return R.fail("数据集id为空");
		}
		R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser());
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = null;//dynamicTableService.selectList(reportDatasource, querySql);
		if (component.getLimitVal() != null && component.getLimitVal() > 0) { // 分页查询
			resultList = dynamicTableService.selectPageList(new Page<>(0, component.getLimitVal()).setSearchCount(false),
					reportDatasource, querySql);
		} else {
			resultList = dynamicTableService.selectList(reportDatasource, querySql);
		}
		List<Map<String, Object>> results = null;
		if (CollectionUtil.isNotEmpty(querySqlParam.getSelectTime()) && CollectionUtil.isNotEmpty(querySqlParam.getHourTime())
				&& StrUtil.isNotEmpty(querySqlParam.getTimeColum())){
			results = new ArrayList<>();
			List<String> selectTime = querySqlParam.getSelectTime();
			List<String> hourTime = querySqlParam.getHourTime();
			String endTime = selectTime.get(1);
			String startTime = selectTime.get(0);
			String startHour = hourTime.get(0);
			String endHour = hourTime.get(1);
			int startDate = Integer.parseInt(startTime + startHour);
			int endDate = Integer.parseInt(endTime + endHour);
			if (startHour.length() < 2){
				startDate = Integer.parseInt(startTime+0+startHour);
			}
			if (endHour.length() < 2){
				endDate = Integer.parseInt(endTime+0+endHour);
			}
			for (Map<String, Object> map : resultList) {
				int date = Integer.parseInt(map.get(querySqlParam.getTimeColum()).toString());
				if (date < startDate ||  date > endDate){
					continue;
				}
				results.add(map);
			}
		} else {
			results = resultList;
		}
		//列权限，删除掉不显示的列
		_formatExcludeColumn(fieldDict, results, selectColumnList, querySqlParam.getExcludeColumnIdList());
		String cubeName = querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null;
		return R.data(new PreviewResult(results, fieldDict, cubeName), querySql);
	}

	private List<Object> getSelectTime(String filterList,String filterColum) {
		JSONArray jsonArray = JSONUtil.parseArray(filterList);
		List<Object> selectTime = new ArrayList<>();
		for (Object obj : jsonArray) {
			JSONObject entries = JSONUtil.parseObj(obj);
			ComponentColumn column = JSON.parseObject(entries.getStr("column"), ComponentColumn.class);
			if (column.getColumnName().equals(filterColum)){
				String conditions = entries.getStr("conditions");
				List<ComponentFilterCondition> filterConditions = JSON.parseArray(conditions, ComponentFilterCondition.class);
				if (filterConditions != null && filterConditions.size() > 0){
					List<Object> selectTimes = new ArrayList<>();
					for (ComponentFilterCondition filterCondition : filterConditions) {
						selectTimes.add(filterCondition.getValues());
					}
					List<Object> objs = (List<Object>) selectTimes.get(0);
					if (objs != null & objs.size() >= 1) {
						selectTime.clear();
						selectTime.addAll(selectTimes);
					}
				}
			}
		}
		return selectTime;
	}

	private String getFilterColum(String columnLists) {
		List<ComponentColumn> columnList = JSON.parseArray(columnLists, ComponentColumn.class);
		if (columnList != null && columnList.size() > 0){
			for (ComponentColumn componentColumn : columnList) {
				ComponentExtJson componentExtJson = JSON.parseObject(componentColumn.getExtJson(), ComponentExtJson.class);
				if (componentExtJson.getIsDateComplete() != null && componentExtJson.getIsDateComplete()){
					return componentColumn.getColumnName();
				}
			}
		}
		return null;
	}

	/**
	 * 带分页的报表查询
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/pagePreview")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "分页报表预览", notes = "传入ComponentDto")
	public R<PreviewDetailResult> dynamicDetailPreview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		//1.先根据维度数据聚合查询分页记录
		VisualComponent component = componentDto.getComponent();
		R<QuerySqlParam> logicPageQuerySqlRet = querySqlService.parseLogicPageQuerySql(component.getCubeDataId(), componentDto, getUser());
		if(!logicPageQuerySqlRet.isSuccess()) {
			return R.fail(logicPageQuerySqlRet.getMsg());
		}
		QuerySqlParam querySqlParam = logicPageQuerySqlRet.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		CubeData cubeData = querySqlParam.getCubeData();
		Map<String, ComponentColumn> weiduColumnMap = querySqlParam.getSelectColumnList().stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, t -> t));
		String logicPageQuerySql = querySqlParam.getQuerySql();
		//维度聚合，查询出所有数据，做逻辑分页
		List<Map<String, Object>> logicDataList = dynamicTableService.selectList(reportDatasource, logicPageQuerySql);
		//页码和每页条数
		int pageNum = componentDto.getPageNum() == null || componentDto.getPageNum() <= 0 ? 1 : componentDto.getPageNum();
		int limitVal = component.getLimitVal() == null || component.getLimitVal() <= 0 ? CubeConstants.DEFAULT_PREVIEW_DATA_SIZE : component.getLimitVal();
		long total = logicDataList.size(), pages = (total + limitVal - 1) / limitVal;
		//取出当前页的维度数据
		List<Map<String, Object>> subDataList = ListUtil.sub(logicDataList, (pageNum - 1) * limitVal, pageNum * limitVal);
		//如果不为空，构造维度数据的where条件
		if(CollectionUtil.isNotEmpty(subDataList)) {
			Map<String, Set<Object>> valueMap = new HashMap<>();
			for(Map<String, Object> dataMap : subDataList) {
				dataMap.forEach((k,v) -> {
					Set<Object> objectSet = valueMap.get(k);
					if(objectSet == null) {
						objectSet = new HashSet<>();
						valueMap.put(k, objectSet);
					}
					objectSet.add(v);
				});
			}
			List<ComponentFilter> filterList = new ArrayList<ComponentFilter>();
			//构造时间字段过滤器
			valueMap.forEach((k,v) -> {
				ComponentFilter timeColumnFilter = new ComponentFilter(weiduColumnMap.get(k));
				timeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.in, new ArrayList<>(v))));
				filterList.add(timeColumnFilter);
			});
			componentDto.setExtFilterJson(JSON.toJSONString(filterList));
		}
		R<QuerySqlParam> r = querySqlService.parseQuerySql(cubeData, reportDatasource, componentDto, getUser(), ComponentTypeEnum.GENERAL);
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		String querySql = r.getData().getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = dynamicTableService.selectList(reportDatasource, querySql);
		PreviewDetailResult previewDetailResult = null;
		if(CollectionUtil.isNotEmpty(resultList)) {
			//列权限，删除掉不显示的列
			_formatExcludeColumn(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList());
			previewDetailResult = new PreviewDetailResult(total, pages, limitVal);
			previewDetailResult.setDataList(resultList);
			previewDetailResult.setFieldDict(fieldDict);
			previewDetailResult.setCubeName(querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null);
			previewDetailResult.setPermStatus(true);
		}
		return R.data(previewDetailResult, querySql);
	}
	
	/**
	 * 列权限校验
	 * @param fieldDict
	 * @param resultList
	 * @param columnList
	 * @param excludeColumnIdList
	 */
	void _formatExcludeColumn(Map<String, String> fieldDict, List<Map<String, Object>> resultList, List<ComponentColumn> columnList, Set<Integer> excludeColumnIdList) {
		//列权限，删除掉不显示的列
		if(CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
			Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, ComponentColumn::getColumnAliasName));
			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
			if(CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
				//1.格式化数据
				for(Map<String, Object> jsonObj : resultList) {
					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
				}
				if(fieldDict != null) { //2.格式化fieldDict
					excludeColumnAliasList.stream().forEach(item -> fieldDict.remove(item));
				}
			}
		}
	}

}

