package org.springblade.visual.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualLoginFreeLinkPermissionDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualLoginFreeLinkService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;


/**
 * @author cyc
 *
 */
@RestController
@RequestMapping("visual/longinfreelink")
@Api(value = "免登陆链接接口", tags = "免登陆链接接口")
public class VisualLoginFreeLinkController extends BladeController {

    @Autowired
    private VisualLoginFreeLinkService loginFreeLinkService;

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private DataPermClient dataPermClient;

    /**
     * 列表
     */
    @GetMapping("/list")
    @ApiOperation(value = "免登陆链接列表")
    @ApiOperationSupport(order = 1)
    public R<IPage<VisualLoginFreeLink>> list(VisualLoginFreeLink visualLoginFreeLink, Query query){
        LambdaQueryWrapper<VisualLoginFreeLink> queryWrapper = Condition.getQueryWrapper(visualLoginFreeLink).lambda();
        Long userId = getUser().getUserId();
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
        // 如果当前登录的人员不是超级管理员则按权限范围内查询
        if(checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> t.eq(VisualLoginFreeLink::getCreateUserId, userId)
                    .or().inSql(VisualLoginFreeLink::getId, "select a.link_id from visual_login_free_link_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                    .or().eq(VisualLoginFreeLink::getPublicStatus, 1));
        }
        if (StrUtil.isNotBlank(visualLoginFreeLink.getVisualDataTitle())) {
            queryWrapper.like(VisualLoginFreeLink::getVisualDataTitle, "%" + visualLoginFreeLink.getVisualDataTitle() + "%");
        }
        queryWrapper.orderByDesc(VisualLoginFreeLink::getId);
        return R.data(loginFreeLinkService.page(Condition.getPage(query), queryWrapper));
    }


    /**
     * 单条数据查询接口
     * @param id
     * @return
     */
    @GetMapping("/detail")
    @ApiOperation(value = "根据ID获取免登陆链接", httpMethod = "GET")
    @ApiOperationSupport(order = 2)
    public R<VisualLoginFreeLink> info(@RequestParam Integer id) {
        return R.data(loginFreeLinkService.getById(id));
    }


    /**
     * 	删除
     * @param ids
     * @return
     */
    @PostMapping("/remove")
    @ApiOperation(value = "根据ID数组删除免登陆链接(多个逗号分割)", httpMethod = "POST")
    @ApiOperationSupport(order = 3)
    public R<String> doRemove(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
        return R.status(loginFreeLinkService.removeByIdList(Func.toIntList(ids)));
    }

    /**
     * 保存
     */
    @PostMapping("/save")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "新增 save", notes = "传入visualLoginFreeLink")
    public R<Integer> save(@Valid @RequestBody VisualLoginFreeLinkPermissionDto linkPermissionDto) {
        VisualData visualData = visualDataService.getById(linkPermissionDto.getVisualDataId());
        if (visualData == null){
            return R.fail("大屏不存在！");
        }

        return R.data(loginFreeLinkService.saveOrUpdateLoginFreeLink(linkPermissionDto,visualData,getUser()));
    }

}
